@@ -1,36 +1,37 @@
 from __future__ import annotations
 
-from typing import List, Tuple, Optional, TYPE_CHECKING
 from dataclasses import dataclass
+from math import isinf
+from typing import TYPE_CHECKING, List, Tuple
+
 import numpy as np
-from matplotlib.colors import ListedColormap
 import triangle
+from matplotlib.colors import ListedColormap
 
 import concreteproperties.utils as utils
+from concreteproperties.material import Concrete
 from concreteproperties.post import plotting_context
 
-import sectionproperties.analysis.fea as sp_fea
-
 if TYPE_CHECKING:
     import matplotlib
-    from concreteproperties.material import Concrete
-    from sectionproperties.pre.geometry import Geometry
+
+    from concreteproperties.material import Material
+    from concreteproperties.pre import CPGeom
 
 
 class AnalysisSection:
-    """Class for an analysis section to perform a fast analysis on concrete sections."""
+    """Class for an analysis section to perform a fast analysis on meshed sections."""
 
     def __init__(
         self,
-        geometry: Geometry,
+        geometry: CPGeom,
     ):
         """Inits the AnalysisSection class.
 
         :param geometry: Geometry object
-        :type geometry: :class:`sectionproperties.pre.geometry.Geometry`
         """
 
-        self.geometry = geometry
+        self.material = geometry.material
 
         # create simple mesh
         tri = {}  # create tri dictionary
@@ -40,6 +41,7 @@ class AnalysisSection:
         if geometry.holes:
             tri["holes"] = geometry.holes  # set holes
 
+        # coarse mesh
         self.mesh = triangle.triangulate(tri, "p")
 
         # extract mesh data
@@ -51,9 +53,9 @@ class AnalysisSection:
             self.mesh_elements = []
 
         # build elements
-        self.elements = []
+        self.elements: List[Tri3] = []
 
-        for idx, node_ids in enumerate(self.mesh_elements):
+        for node_ids in self.mesh_elements:
             x1 = self.mesh_nodes[node_ids[0]][0]
             y1 = self.mesh_nodes[node_ids[0]][1]
             x2 = self.mesh_nodes[node_ids[1]][0]
@@ -67,10 +69,9 @@ class AnalysisSection:
             # add tri elements to the mesh
             self.elements.append(
                 Tri3(
-                    el_id=idx,
                     coords=coords,
                     node_ids=node_ids,
-                    conc_material=self.geometry.material,
+                    material=self.material,
                 )
             )
 
@@ -88,19 +89,18 @@ class AnalysisSection:
     ) -> Tuple[np.ndarray, float, float, float]:
         r"""Given section actions and section propreties, calculates elastic stresses.
 
-        :param float n: Axial force
-        :param float m_x: Bending moment about the x-axis
-        :param float m_y: Bending moment about the y-axis
-        :param float e_a: Axial rigidity
-        :param float cx: x-Centroid
-        :param float cy: y-Centroid
-        :param float e_ixx: Flexural rigidity about the x-axis
-        :param float e_iyy: Flexural rigidity about the y-axis
-        :param float e_ixy: Flexural rigidity about the xy-axis
+        :param n: Axial force
+        :param m_x: Bending moment about the x-axis
+        :param m_y: Bending moment about the y-axis
+        :param e_a: Axial rigidity
+        :param cx: x-Centroid
+        :param cy: y-Centroid
+        :param e_ixx: Flexural rigidity about the x-axis
+        :param e_iyy: Flexural rigidity about the y-axis
+        :param e_ixy: Flexural rigidity about the xy-axis
 
         :return: Elastic stresses, net force and distance from neutral axis to point of
             force action
-        :rtype: Tuple[:class:`numpy.ndarray`, float, float, float]
         """
 
         # intialise stress results
@@ -112,22 +112,22 @@ class AnalysisSection:
             y = node[1] - cy
 
             # axial stress
-            sig[idx] += n * self.geometry.material.elastic_modulus / e_a
+            sig[idx] += n * self.material.elastic_modulus / e_a
 
             # bending moment
-            sig[idx] += self.geometry.material.elastic_modulus * (
+            sig[idx] += self.material.elastic_modulus * (
                 -(e_ixy * m_x) / (e_ixx * e_iyy - e_ixy**2) * x
                 + (e_iyy * m_x) / (e_ixx * e_iyy - e_ixy**2) * y
             )
-            sig[idx] += self.geometry.material.elastic_modulus * (
+            sig[idx] += self.material.elastic_modulus * (
                 +(e_ixx * m_y) / (e_ixx * e_iyy - e_ixy**2) * x
                 - (e_ixy * m_y) / (e_ixx * e_iyy - e_ixy**2) * y
             )
 
         # initialise section actions
-        n_conc = 0
-        m_x_conc = 0
-        m_y_conc = 0
+        n_sec = 0
+        m_x_sec = 0
+        m_y_sec = 0
 
         for el in self.elements:
             el_n, el_m_x, el_m_y = el.calculate_elastic_actions(
@@ -142,90 +142,88 @@ class AnalysisSection:
                 e_ixy=e_ixy,
             )
 
-            n_conc += el_n
-            m_x_conc += el_m_x
-            m_y_conc += el_m_y
+            n_sec += el_n
+            m_x_sec += el_m_x
+            m_y_sec += el_m_y
 
         # calculate point of action
-        if n_conc == 0:
+        if n_sec == 0:
             d_x = 0
             d_y = 0
         else:
-            d_x = m_y_conc / n_conc
-            d_y = m_x_conc / n_conc
+            d_x = m_y_sec / n_sec
+            d_y = m_x_sec / n_sec
 
-        return sig, n_conc, d_x, d_y
+        return sig, n_sec, d_x, d_y
 
-    def service_stress_analysis(
+    def service_analysis(
         self,
-        point_na: Tuple[float],
-        d_n: float,
+        point_na: Tuple[float, float],
         theta: float,
         kappa: float,
-        centroid: Tuple[float],
-    ) -> Tuple[float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[float, float, float, float, float]:
         r"""Performs a service stress analysis on the section.
 
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float d_n: Depth of the neutral axis from the extreme compression fibre
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param d_n: Depth of the neutral axis from the extreme compression fibre
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
-        :param float kappa: Curvature
+        :param kappa: Curvature
         :param centroid: Centroid about which to take moments
-        :type centroid: Tuple[float]
 
-        :return: Axial force, section moments and max strain
-        :rtype: Tuple[float]
+        :return: Axial force, section moments and min/max strain
         """
 
         # initialise section actions
-        n = 0
+        n_sec = 0
+        m_x_sec = 0
+        m_y_sec = 0
+        min_strain = 0
         max_strain = 0
-        m_x = 0
-        m_y = 0
-        m_v = 0
 
         for el in self.elements:
-            el_n, el_m_x, el_m_y, el_m_v, el_max_strain = el.calculate_service_actions(
+            (
+                el_n,
+                el_m_x,
+                el_m_y,
+                el_min_strain,
+                el_max_strain,
+            ) = el.calculate_service_actions(
                 point_na=point_na,
-                d_n=d_n,
                 theta=theta,
                 kappa=kappa,
                 centroid=centroid,
             )
+            min_strain = min(min_strain, el_min_strain)
             max_strain = max(max_strain, el_max_strain)
 
-            n += el_n
-            m_x += el_m_x
-            m_y += el_m_y
-            m_v += el_m_v
+            n_sec += el_n
+            m_x_sec += el_m_x
+            m_y_sec += el_m_y
 
-        return n, m_x, m_y, m_v, max_strain
+        return n_sec, m_x_sec, m_y_sec, min_strain, max_strain
 
     def get_service_stress(
         self,
         d_n: float,
         kappa: float,
-        point_na: Tuple[float],
+        point_na: Tuple[float, float],
         theta: float,
-        centroid: Tuple[float],
-    ) -> Tuple[np.ndarray, float, float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[np.ndarray, float, float, float]:
         r"""Given the neutral axis depth `d_n` and curvature `kappa` determines the
         service stresses within the section.
 
-        :param float d_n: Neutral axis depth
-        :param float kappa: Curvature
+        :param d_n: Neutral axis depth
+        :param kappa: Curvature
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
         :param centroid: Centroid about which to take moments
-        :type centroid: Tuple[float]
 
         :return: Service stresses, net force and distance from centroid to point of
             force action
-        :rtype: Tuple[:class:`numpy.ndarray`, float, float, float]
         """
 
         # intialise stress results
@@ -242,56 +240,50 @@ class AnalysisSection:
             )
 
             # get stress at gauss point
-            sig[idx] = self.geometry.material.stress_strain_profile.get_stress(
-                strain=strain
-            )
+            sig[idx] = self.material.stress_strain_profile.get_stress(strain=strain)
 
         # calculate total force
-        n, m_x, m_y, _, _ = self.service_stress_analysis(
+        n_sec, m_x_sec, m_y_sec, _, _ = self.service_analysis(
             point_na=point_na,
-            d_n=d_n,
             theta=theta,
             kappa=kappa,
             centroid=centroid,
         )
 
         # calculate point of action
-        if n == 0:
+        if n_sec == 0:
             d_x = 0
             d_y = 0
         else:
-            d_x = m_y / n
-            d_y = m_x / n
+            d_x = m_y_sec / n_sec
+            d_y = m_x_sec / n_sec
 
-        return sig, n, d_x, d_y
+        return sig, n_sec, d_x, d_y
 
-    def ultimate_stress_analysis(
+    def ultimate_analysis(
         self,
-        point_na: Tuple[float],
+        point_na: Tuple[float, float],
         d_n: float,
         theta: float,
         ultimate_strain: float,
-        pc: Tuple[float],
-    ) -> Tuple[float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[float, float, float]:
         r"""Performs an ultimate stress analysis on the section.
 
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float d_n: Depth of the neutral axis from the extreme compression fibre
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param d_n: Depth of the neutral axis from the extreme compression fibre
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
-        :param float ultimate_strain: Concrete strain at failure
-        :param pc: Location of the plastic centroid
-        :type pc: Tuple[float]
+        :param ultimate_strain: Concrete strain at failure
+        :param centroid: Centroid about which to take moments
 
         :return: Axial force and resultant moments about the global axes
-        :rtype: Tuple[float]
         """
 
         # initialise section actions
-        n = 0
-        m_x = 0
-        m_y = 0
+        n_sec = 0
+        m_x_sec = 0
+        m_y_sec = 0
 
         for el in self.elements:
             el_n, el_m_x, el_m_y = el.calculate_ultimate_actions(
@@ -299,38 +291,35 @@ class AnalysisSection:
                 d_n=d_n,
                 theta=theta,
                 ultimate_strain=ultimate_strain,
-                pc=pc,
+                centroid=centroid,
             )
 
-            n += el_n
-            m_x += el_m_x
-            m_y += el_m_y
+            n_sec += el_n
+            m_x_sec += el_m_x
+            m_y_sec += el_m_y
 
-        return n, m_x, m_y
+        return n_sec, m_x_sec, m_y_sec
 
     def get_ultimate_stress(
         self,
         d_n: float,
-        point_na: Tuple[float],
+        point_na: Tuple[float, float],
         theta: float,
         ultimate_strain: float,
-        pc: Tuple[float],
-    ) -> Tuple[np.ndarray, float, float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[np.ndarray, float, float, float]:
         r"""Given the neutral axis depth `d_n` and ultimate strain, determines the
         ultimate stresses with the section.
 
-        :param float d_n: Neutral axis depth
+        :param d_n: Neutral axis depth
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
-        :param float ultimate_strain: Concrete strain at failure
-        :param pc: Location of the plastic centroid
-        :type pc: Tuple[float]
+        :param ultimate_strain: Concrete strain at failure
+        :param centroid: Centroid about which to take moments
 
         :return: Ultimate stresses net force and distance from neutral axis to point of
             force action
-        :rtype: Tuple[:class:`numpy.ndarray`, float, float, float]
         """
 
         # intialise stress results
@@ -339,54 +328,57 @@ class AnalysisSection:
         # loop through nodes
         for idx, node in enumerate(self.mesh_nodes):
             # get strain at node
-            strain = utils.get_ultimate_strain(
-                point=(node[0], node[1]),
-                point_na=point_na,
-                d_n=d_n,
-                theta=theta,
-                ultimate_strain=ultimate_strain,
-            )
+            if isinf(d_n):
+                strain = ultimate_strain
+            else:
+                strain = utils.get_ultimate_strain(
+                    point=(node[0], node[1]),
+                    point_na=point_na,
+                    d_n=d_n,
+                    theta=theta,
+                    ultimate_strain=ultimate_strain,
+                )
 
             # get stress at gauss point
-            sig[idx] = self.geometry.material.ultimate_stress_strain_profile.get_stress(
-                strain=strain
-            )
+            if isinstance(self.material, Concrete):
+                sig[idx] = self.material.ultimate_stress_strain_profile.get_stress(
+                    strain=strain
+                )
+            else:
+                sig[idx] = self.material.stress_strain_profile.get_stress(strain=strain)
 
         # calculate total force
-        n, m_x, m_y = self.ultimate_stress_analysis(
+        n_sec, m_x_sec, m_y_sec = self.ultimate_analysis(
             point_na=point_na,
             d_n=d_n,
             theta=theta,
             ultimate_strain=ultimate_strain,
-            pc=pc,
+            centroid=centroid,
         )
 
         # calculate point of action
-        if n == 0:
+        if n_sec == 0:
             d_x = 0
             d_y = 0
         else:
-            d_x = m_y / n
-            d_y = m_x / n
+            d_x = m_y_sec / n_sec
+            d_y = m_x_sec / n_sec
 
-        return sig, n, d_x, d_y
+        return sig, n_sec, d_x, d_y
 
     def plot_mesh(
         self,
-        alpha: Optional[float] = 0.5,
-        title: Optional[str] = "Finite Element Mesh",
+        alpha: float = 0.5,
+        title: str = "Finite Element Mesh",
         **kwargs,
-    ) -> matplotlib.axes.Axes:
+    ) -> matplotlib.axes.Axes:  # type: ignore
         """Plots the finite element mesh.
 
         :param alpha: Transparency of the mesh outlines
-        :type alpha: Optional[float]
         :param title: Plot title
-        :type title: Optional[str]
         :param kwargs: Passed to :func:`~concreteproperties.post.plotting_context`
 
         :return: Matplotlib axes object
-        :rtype: :class:`matplotlib.axes.Axes`
         """
 
         with plotting_context(title=title, **kwargs) as (fig, ax):
@@ -394,51 +386,50 @@ class AnalysisSection:
             c = []  # Indices of elements for mapping colours
 
             # create an array of finite element colours
-            for idx, element in enumerate(self.elements):
-                colour_array.append(element.conc_material.colour)
+            for idx, el in enumerate(self.elements):
+                colour_array.append(el.material.colour)
                 c.append(idx)
 
             cmap = ListedColormap(colour_array)  # custom colourmap
 
             # plot the mesh colours
-            ax.tripcolor(
+            ax.tripcolor(  # type: ignore
                 self.mesh_nodes[:, 0],
                 self.mesh_nodes[:, 1],
-                self.mesh_elements[:, 0:3],
+                self.mesh_elements[:, 0:3],  # type: ignore
                 c,
                 cmap=cmap,
             )
 
             # plot the mesh
-            ax.triplot(
+            ax.triplot(  # type: ignore
                 self.mesh_nodes[:, 0],
                 self.mesh_nodes[:, 1],
-                self.mesh_elements[:, 0:3],
+                self.mesh_elements[:, 0:3],  # type: ignore
                 lw=0.5,
                 color="black",
                 alpha=alpha,
             )
 
-            ax.set_aspect("equal", anchor="C")
+            ax.set_aspect("equal", anchor="C")  # type: ignore
 
         return ax
 
     def plot_shape(
         self,
-        ax: matplotlib.axes.Axes,
+        ax: matplotlib.axes.Axes,  # type: ignore
     ):
         """Plots the coloured shape of the mesh with no outlines on `ax`.
 
         :param ax: Matplotlib axes object
-        :type ax: :class:`matplotlib.axes.Axes`
         """
 
         colour_array = []
         c = []  # Indices of elements for mapping colours
 
         # create an array of finite element colours
-        for idx, element in enumerate(self.elements):
-            colour_array.append(element.conc_material.colour)
+        for idx, el in enumerate(self.elements):
+            colour_array.append(el.material.colour)
             c.append(idx)
 
         cmap = ListedColormap(colour_array)  # custom colourmap
@@ -447,7 +438,7 @@ class AnalysisSection:
         ax.tripcolor(
             self.mesh_nodes[:, 0],
             self.mesh_nodes[:, 1],
-            self.mesh_elements[:, 0:3],
+            self.mesh_elements[:, 0:3],  # type: ignore
             c,
             cmap=cmap,
         )
@@ -457,27 +448,21 @@ class AnalysisSection:
 class Tri3:
     """Class for a three noded linear triangular element.
 
-    :param int el_id: Unique element id
-    :param coords: A 2 x 3 array of the coordinates of the tri-3 nodes.
-    :type coords: :class:`numpy.ndarray`
+    :param coords: A 2 x 3 array of the coordinates of the tri-3 nodes
     :param node_ids: A list of the global node ids for the current element
-    :type node_ids: List[int]
-    :param conc_material: Material object for the current finite element.
-    :type conc_material: :class:`~concreteproperties.material.Concrete`
+    :param material: Material object for the current finite element
     """
 
-    el_id: int
     coords: np.ndarray
     node_ids: List[int]
-    conc_material: Concrete
+    material: Material
 
     def second_moments_of_area(
         self,
-    ) -> Tuple[float]:
+    ) -> Tuple[float, float, float]:
         """Calculates the second moments of area for the current finite element.
 
         :return: Modulus weighted second moments of area *(e_ixx, e_iyy, e_ixy)*
-        :rtype: Tuple[float]
         """
 
         # initialise properties
@@ -494,19 +479,19 @@ class Tri3:
             N, j = utils.shape_function(coords=self.coords, gauss_point=gp)
 
             e_ixx += (
-                self.conc_material.elastic_modulus
+                self.material.elastic_modulus
                 * gp[0]
                 * np.dot(N, np.transpose(self.coords[1, :])) ** 2
                 * j
             )
             e_iyy += (
-                self.conc_material.elastic_modulus
+                self.material.elastic_modulus
                 * gp[0]
                 * np.dot(N, np.transpose(self.coords[0, :])) ** 2
                 * j
             )
             e_ixy += (
-                self.conc_material.elastic_modulus
+                self.material.elastic_modulus
                 * gp[0]
                 * np.dot(N, np.transpose(self.coords[1, :]))
                 * np.dot(N, np.transpose(self.coords[0, :]))
@@ -526,21 +511,20 @@ class Tri3:
         e_ixx: float,
         e_iyy: float,
         e_ixy: float,
-    ) -> Tuple[float]:
-        r"""Calculates elastic actions for the current finite element.
+    ) -> Tuple[float, float, float]:
+        """Calculates elastic actions for the current finite element.
 
-        :param float n: Axial force
-        :param float m_x: Bending moment about the x-axis
-        :param float m_y: Bending moment about the y-axis
-        :param float e_a: Axial rigidity
-        :param float cx: x-Centroid
-        :param float cy: y-Centroid
-        :param float e_ixx: Flexural rigidity about the x-axis
-        :param float e_iyy: Flexural rigidity about the y-axis
-        :param float e_ixy: Flexural rigidity about the xy-axis
+        :param n: Axial force
+        :param m_x: Bending moment about the x-axis
+        :param m_y: Bending moment about the y-axis
+        :param e_a: Axial rigidity
+        :param cx: x-Centroid
+        :param cy: y-Centroid
+        :param e_ixx: Flexural rigidity about the x-axis
+        :param e_iyy: Flexural rigidity about the y-axis
+        :param e_ixy: Flexural rigidity about the xy-axis
 
-        :return: Elastic force and resultant moment
-        :rtype: Tuple[float]
+        :return: Elastic force and resultant moments
         """
 
         # initialise element results
@@ -562,12 +546,12 @@ class Tri3:
 
             # axial force
             force_gp = 0
-            force_gp += gp[0] * n * self.conc_material.elastic_modulus / e_a * j
+            force_gp += gp[0] * n * self.material.elastic_modulus / e_a * j
 
             # bending moment
             force_gp += (
                 gp[0]
-                * self.conc_material.elastic_modulus
+                * self.material.elastic_modulus
                 * (
                     -(e_ixy * m_x) / (e_ixx * e_iyy - e_ixy**2) * x
                     + (e_iyy * m_x) / (e_ixx * e_iyy - e_ixy**2) * y
@@ -576,7 +560,7 @@ class Tri3:
             )
             force_gp += (
                 gp[0]
-                * self.conc_material.elastic_modulus
+                * self.material.elastic_modulus
                 * (
                     +(e_ixx * m_y) / (e_ixx * e_iyy - e_ixy**2) * x
                     - (e_ixy * m_y) / (e_ixx * e_iyy - e_ixy**2) * y
@@ -593,33 +577,28 @@ class Tri3:
 
     def calculate_service_actions(
         self,
-        point_na: Tuple[float],
-        d_n: float,
+        point_na: Tuple[float, float],
         theta: float,
         kappa: float,
-        centroid: Tuple[float],
-    ) -> Tuple[float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[float, float, float, float, float]:
         r"""Calculates service actions for the current finite element.
 
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float d_n: Depth of the neutral axis from the extreme compression fibre
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
-        :param float kappa: Curvature
+        :param kappa: Curvature
         :param centroid: Centroid about which to take moments
-        :type centroid: Tuple[float]
 
-        :return: Axial force, moments and maximum strain
-        :rtype: Tuple[float]
+        :return: Axial force, moments and min/max strain
         """
 
         # initialise element results
         force_e = 0
-        max_strain_e = 0
         m_x_e = 0
         m_y_e = 0
-        m_v_e = 0
+        min_strain_e = 0
+        max_strain_e = 0
 
         # get points for 1 point Gaussian integration
         gps = utils.gauss_points(n=1)
@@ -640,45 +619,40 @@ class Tri3:
                 theta=theta,
                 kappa=kappa,
             )
+            min_strain_e = min(min_strain_e, strain)
             max_strain_e = max(max_strain_e, strain)
 
             # get stress at gauss point
-            stress = self.conc_material.stress_strain_profile.get_stress(strain=strain)
+            stress = self.material.stress_strain_profile.get_stress(strain=strain)
 
             # calculate force (stress * area)
             force_gp = gp[0] * stress * j
 
-            # convert gauss point to local coordinates
-            u, _ = sp_fea.principal_coordinate(phi=theta * 180 / np.pi, x=x, y=y)
             # add force and moment
             force_e += force_gp
             m_x_e += force_e * (y - centroid[1])
             m_y_e += force_e * (x - centroid[0])
-            m_v_e += force_gp * u
 
-        return force_e, m_x_e, m_y_e, m_v_e, max_strain_e
+        return force_e, m_x_e, m_y_e, min_strain_e, max_strain_e
 
     def calculate_ultimate_actions(
         self,
-        point_na: Tuple[float],
+        point_na: Tuple[float, float],
         d_n: float,
         theta: float,
         ultimate_strain: float,
-        pc: Tuple[float],
-    ) -> Tuple[float]:
+        centroid: Tuple[float, float],
+    ) -> Tuple[float, float, float]:
         r"""Calculates ultimate actions for the current finite element.
 
         :param point_na: Point on the neutral axis
-        :type point_na: Tuple[float]
-        :param float d_n: Depth of the neutral axis from the extreme compression fibre
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param d_n: Depth of the neutral axis from the extreme compression fibre
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
-        :param float ultimate_strain: Concrete strain at failure
-        :param pc: Location of the plastic centroid
-        :type pc: Tuple[float]
+        :param ultimate_strain: Concrete strain at failure
+        :param centroid: Centroid about which to take moments
 
-        :return: Axial force and resultant moment about the global axes
-        :rtype: Tuple[float]
+        :return: Axial force and resultant moments about the global axes
         """
 
         # initialise element results
@@ -699,25 +673,31 @@ class Tri3:
             y = np.dot(N, np.transpose(self.coords[1, :]))
 
             # get strain at gauss point
-            strain = utils.get_ultimate_strain(
-                point=(x, y),
-                point_na=point_na,
-                d_n=d_n,
-                theta=theta,
-                ultimate_strain=ultimate_strain,
-            )
+            if isinf(d_n):
+                strain = ultimate_strain
+            else:
+                strain = utils.get_ultimate_strain(
+                    point=(x, y),
+                    point_na=point_na,
+                    d_n=d_n,
+                    theta=theta,
+                    ultimate_strain=ultimate_strain,
+                )
 
             # get stress at gauss point
-            stress = self.conc_material.ultimate_stress_strain_profile.get_stress(
-                strain=strain
-            )
+            if isinstance(self.material, Concrete):
+                stress = self.material.ultimate_stress_strain_profile.get_stress(
+                    strain=strain
+                )
+            else:
+                stress = self.material.stress_strain_profile.get_stress(strain=strain)
 
             # calculate force (stress * area)
             force_gp = gp[0] * stress * j
 
             # add force and moment
             force_e += force_gp
-            m_x_e += force_gp * (y - pc[1])
-            m_y_e += force_gp * (x - pc[0])
+            m_x_e += force_gp * (y - centroid[1])
+            m_y_e += force_gp * (x - centroid[0])
 
         return force_e, m_x_e, m_y_e
