@@ -1,18 +1,19 @@
 from __future__ import annotations
 
-from typing import Optional, TYPE_CHECKING
 from copy import deepcopy
+from math import inf
+from multiprocessing.sharedctypes import Value
+from typing import TYPE_CHECKING, List, Tuple
+
 import numpy as np
+from rich.live import Live
 from scipy.interpolate import interp1d
 from scipy.optimize import brentq
-from rich.live import Live
 
-from concreteproperties.material import Concrete, Steel
-import concreteproperties.stress_strain_profile as ssp
 import concreteproperties.results as res
+import concreteproperties.stress_strain_profile as ssp
 import concreteproperties.utils as utils
-
-from sectionproperties.analysis.fea import principal_coordinate
+from concreteproperties.material import Concrete, SteelBar
 
 if TYPE_CHECKING:
     from concreteproperties.concrete_section import ConcreteSection
@@ -35,8 +36,6 @@ class DesignCode:
         """Assigns a concrete section to the design code.
 
         :param concrete_section: Concrete section object to analyse
-        :type concrete_section:
-            :class:`~concreteproperties.concrete_section.ConcreteSection`
         """
 
         self.concrete_section = concrete_section
@@ -44,18 +43,16 @@ class DesignCode:
     def create_concrete_material(
         self,
         compressive_strength: float,
-        colour: Optional[str] = "lightgrey",
+        colour: str = "lightgrey",
     ) -> Concrete:
         """Returns a concrete material object.
 
         List assumptions of material properties here...
 
-        :param float compressive_strength: Concrete compressive strength
+        :param compressive_strength: Concrete compressive strength
         :param colour: Colour of the concrete for rendering
-        :type colour: Optional[str]
 
         :return: Concrete material object
-        :rtype: :class:`~concreteproperties.material.Concrete`
         """
 
         raise NotImplementedError
@@ -63,18 +60,16 @@ class DesignCode:
     def create_steel_material(
         self,
         yield_strength: float,
-        colour: Optional[str] = "grey",
-    ) -> Steel:
-        """Returns a steel material object.
+        colour: str = "grey",
+    ) -> SteelBar:
+        """Returns a steel bar material object.
 
         List assumptions of material properties here...
 
-        :param float yield_strength: Steel yield strength
+        :param yield_strength: Steel yield strength
         :param colour: Colour of the steel for rendering
-        :type colour: Optional[str]
 
         :return: Steel material object
-        :rtype: :class:`~concreteproperties.material.Steel`
         """
 
         raise NotImplementedError
@@ -82,14 +77,13 @@ class DesignCode:
     def get_gross_properties(
         self,
         **kwargs,
-    ) -> res.ConcreteProperties:
+    ) -> res.GrossProperties:
         """Returns the gross section properties of the reinforced concrete section.
 
         :param kwargs: Keyword arguments passed to
             :meth:`~concreteproperties.concrete_section.ConcreteSection.get_gross_properties`
 
         :return: Concrete properties object
-        :rtype: :class:`~concreteproperties.results.ConcreteProperties`
         """
 
         return self.concrete_section.get_gross_properties(**kwargs)
@@ -97,15 +91,13 @@ class DesignCode:
     def get_transformed_gross_properties(
         self,
         **kwargs,
-    ) -> res.TransformedConcreteProperties:
+    ) -> res.TransformedGrossProperties:
         """Transforms gross section properties.
 
         :param kwargs: Keyword arguments passed to
             :meth:`~concreteproperties.concrete_section.ConcreteSection.get_transformed_gross_properties`
 
         :return: Transformed concrete properties object
-        :rtype:
-            :class:`~concreteproperties.results.TransformedConcreteProperties`
         """
 
         return self.concrete_section.get_transformed_gross_properties(**kwargs)
@@ -120,7 +112,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.calculate_cracked_properties`
 
         :return: Cracked results object
-        :rtype: :class:`~concreteproperties.results.CrackedResults`
         """
 
         return self.concrete_section.calculate_cracked_properties(**kwargs)
@@ -136,7 +127,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.moment_curvature_analysis`
 
         :return: Moment curvature results object
-        :rtype: :class:`~concreteproperties.results.MomentCurvatureResults`
         """
 
         return self.concrete_section.moment_curvature_analysis(**kwargs)
@@ -151,7 +141,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.ultimate_bending_capacity`
 
         :return: Ultimate bending results object
-        :rtype: :class:`~concreteproperties.results.UltimateBendingResults`
         """
 
         return self.concrete_section.ultimate_bending_capacity(**kwargs)
@@ -166,7 +155,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.moment_interaction_diagram`
 
         :return: Moment interaction results object
-        :rtype: :class:`~concreteproperties.results.MomentInteractionResults`
         """
 
         return self.concrete_section.moment_interaction_diagram(**kwargs)
@@ -181,7 +169,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.biaxial_bending_diagram`
 
         :return: Biaxial bending results
-        :rtype: :class:`~concreteproperties.results.BiaxialBendingResults`
         """
 
         return self.concrete_section.biaxial_bending_diagram(**kwargs)
@@ -197,7 +184,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.calculate_uncracked_stress`
 
         :return: Stress results object
-        :rtype: :class:`~concreteproperties.results.StressResult`
         """
 
         return self.concrete_section.calculate_uncracked_stress(**kwargs)
@@ -213,7 +199,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.calculate_cracked_stress`
 
         :return: Stress results object
-        :rtype: :class:`~concreteproperties.results.StressResult`
         """
 
         return self.concrete_section.calculate_cracked_stress(**kwargs)
@@ -228,7 +213,6 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.calculate_service_stress`
 
         :return: Stress results object
-        :rtype: :class:`~concreteproperties.results.StressResult`
         """
 
         return self.concrete_section.calculate_service_stress(**kwargs)
@@ -243,14 +227,19 @@ class DesignCode:
             :meth:`~concreteproperties.concrete_section.ConcreteSection.calculate_ultimate_stress`
 
         :return: Stress results object
-        :rtype: :class:`~concreteproperties.results.StressResult`
         """
 
         return self.concrete_section.calculate_ultimate_stress(**kwargs)
 
 
 class AS3600(DesignCode):
-    """Design code class for Australian standard AS 3600:2018."""
+    """Design code class for Australian standard AS 3600:2018.
+
+    Note that this design code only supports :class:`~concreteproperties.pre.Concrete`
+    and :class:`~concreteproperties.pre.SteelBar` material objects. Meshed
+    :class:`~concreteproperties.pre.Steel` material objects are **not** supported
+    as this falls under the composite structures design code.
+    """
 
     def __init__(
         self,
@@ -266,30 +255,39 @@ class AS3600(DesignCode):
         """Assigns a concrete section to the design code.
 
         :param concrete_section: Concrete section object to analyse
-        :type concrete_section:
-            :class:`~concreteproperties.concrete_section.ConcreteSection`
         """
 
         self.concrete_section = concrete_section
 
+        # check to make sure there are no meshed reinforcement regions
+        if self.concrete_section.reinf_geometries_meshed:
+            raise ValueError(
+                "Meshed reinforcement is not supported in this design code."
+            )
+
         # determine reinforcement class
         self.reinforcement_class = "N"
 
-        for steel_geom in self.concrete_section.steel_geometries:
-            if steel_geom.material.stress_strain_profile.fracture_strain < 0.05:
+        for steel_geom in self.concrete_section.reinf_geometries_lumped:
+            if (
+                abs(
+                    steel_geom.material.stress_strain_profile.get_ultimate_tensile_strain()
+                )
+                < 0.05
+            ):
                 self.reinforcement_class = "L"
 
-        # re-run plastic property calculation with max compressive strain
-        self.concrete_section.calculate_gross_plastic_properties(
-            max_compressive_strain=0.0025
-        )
+        # calculate squash and tensile load
+        squash, tensile = self.squash_tensile_load()
+        self.squash_load = squash
+        self.tensile_load = tensile
 
     def create_concrete_material(
         self,
         compressive_strength: float,
-        colour: Optional[str] = "lightgrey",
+        colour: str = "lightgrey",
     ) -> Concrete:
-        """Returns a concrete material object to AS 3600:2018.
+        r"""Returns a concrete material object to AS 3600:2018.
 
         | **Material assumptions:**
         | - *Density*: 2400 kg/m\ :sup:`3`
@@ -301,15 +299,14 @@ class AS3600(DesignCode):
         | - *Alpha squash*: From Cl. 10.6.2.2
         | - *Flexural tensile strength*: From Cl. 3.1.1.3
 
-        :param float compressive_strength: Characteristic compressive strength of
+        :param compressive_strength: Characteristic compressive strength of
             concrete at 28 days in megapascals (MPa)
         :param colour: Colour of the concrete for rendering
-        :type colour: Optional[str]
 
-        :raises ValueError: If compressive_strength is not between 20 MPa and 100 MPa.
+        :raises ValueError: If ``compressive_strength`` is not between 20 MPa and
+            100 MPa.
 
         :return: Concrete material object
-        :rtype: :class:`~concreteproperties.material.Concrete`
         """
 
         if compressive_strength < 20 or compressive_strength > 100:
@@ -361,11 +358,11 @@ class AS3600(DesignCode):
 
     def create_steel_material(
         self,
-        yield_strength: Optional[float] = 500,
-        ductility_class: Optional[str] = "N",
-        colour: Optional[str] = "grey",
-    ) -> Steel:
-        """Returns a steel material object.
+        yield_strength: float = 500,
+        ductility_class: str = "N",
+        colour: str = "grey",
+    ) -> SteelBar:
+        r"""Returns a steel bar material object.
 
         | **Material assumptions:**
         | - *Density*: 7850 kg/m\ :sup:`3`
@@ -373,16 +370,12 @@ class AS3600(DesignCode):
         | - *Stress-strain profile:* Elastic-plastic, fracture strain from Table 3.2.1
 
         :param yield_strength: Steel yield strength
-        :type yield_strength: Optional[float]
         :param ductility_class: Steel ductility class ("N" or "L")
-        :type ductility_class: Optional[str]
         :param colour: Colour of the steel for rendering
-        :type colour: Optional[str]
 
-        :raises ValueError: If ductility_class is not N or L
+        :raises ValueError: If ``ductility_class`` is not "N" or "L"
 
         :return: Steel material object
-        :rtype: :class:`~concreteproperties.material.Steel`
         """
 
         if ductility_class == "N":
@@ -392,7 +385,7 @@ class AS3600(DesignCode):
         else:
             raise ValueError("ductility_class must be N or L.")
 
-        return Steel(
+        return SteelBar(
             name=f"{yield_strength:.0f} MPa Steel (AS 3600:2018)",
             density=7.85e-6,
             stress_strain_profile=ssp.SteelElasticPlastic(
@@ -403,6 +396,53 @@ class AS3600(DesignCode):
             colour=colour,
         )
 
+    def squash_tensile_load(
+        self,
+    ) -> Tuple[float, float]:
+        """Calculates the squash and tensile load of the reinforced concrete section.
+
+        :return: Squash and tensile load
+        """
+
+        # initialise the squash load, tensile load and squash moment variables
+        squash_load = 0
+        tensile_load = 0
+
+        # loop through all concrete geometries
+        for conc_geom in self.concrete_section.concrete_geometries:
+            # calculate area and centroid
+            area = conc_geom.calculate_area()
+
+            # calculate compressive force
+            force_c = (
+                area
+                * conc_geom.material.alpha_squash
+                * conc_geom.material.ultimate_stress_strain_profile.get_compressive_strength()
+            )
+
+            # add to totals
+            squash_load += force_c
+
+        # loop through all steel geometries
+        for steel_geom in self.concrete_section.reinf_geometries_lumped:
+            # calculate area and centroid
+            area = steel_geom.calculate_area()
+
+            # calculate compressive and tensile force
+            force_c = area * steel_geom.material.stress_strain_profile.get_stress(
+                strain=0.025
+            )
+
+            force_t = (
+                -area * steel_geom.material.stress_strain_profile.get_yield_strength()
+            )
+
+            # add to totals
+            squash_load += force_c
+            tensile_load += force_t
+
+        return squash_load, tensile_load
+
     def capacity_reduction_factor(
         self,
         n_u: float,
@@ -416,14 +456,13 @@ class AS3600(DesignCode):
         ``n_ub`` and ``phi_0`` only required for compression, ``n_uot`` only required
         for tension.
 
-        :param float n_u: Axial force in member
-        :param float n_ub: Axial force at balanced point
-        :param float n_uot: Axial force at ultimate tension load
-        :param float k_uo: Neutral axis parameter at pure bending
-        :param float phi_0: Capacity reduction factor for dominant compression
+        :param n_u: Axial force in member
+        :param n_ub: Axial force at balanced point
+        :param n_uot: Axial force at ultimate tension load
+        :param k_uo: Neutral axis parameter at pure bending
+        :param phi_0: Capacity reduction factor for dominant compression
 
         :return: Capacity reduction factor
-        :rtype: float
         """
 
         # pure bending phi
@@ -453,11 +492,10 @@ class AS3600(DesignCode):
     ) -> float:
         r"""Returns k_uo for the reinforced concrete cross-section given ``theta``.
 
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
 
         :return: Bending parameter k_uo
-        :rtype: float
         """
 
         pure_res = self.concrete_section.ultimate_bending_capacity(theta=theta)
@@ -470,56 +508,22 @@ class AS3600(DesignCode):
     ) -> float:
         r"""Returns n_ub for the reinforced concrete cross-section given ``theta``.
 
-        :param float theta: Angle (in radians) the neutral axis makes with the
+        :param theta: Angle (in radians) the neutral axis makes with the
             horizontal axis (:math:`-\pi \leq \theta \leq \pi`)
 
         :return: Balanced axial force n_ub
-        :rtype: float
         """
 
-        # 1) find d_0
-        d_0 = 0
-        extreme_geom = None
+        # get depth to extreme tensile bar and its yield strain
+        d_0, eps_sy = self.concrete_section.extreme_bar(theta=theta)
 
-        # calculate extreme fibre in global coordinates
-        extreme_fibre, _ = utils.calculate_extreme_fibre(
-            points=self.concrete_section.geometry.points, theta=theta
-        )
+        # get compressive strain at extreme fibre
+        eps_cu = self.concrete_section.gross_properties.conc_ultimate_strain
 
-        # get depth to extreme tensile steel
-        for steel_geom in self.concrete_section.steel_geometries:
-            centroid = steel_geom.calculate_centroid()
+        # calculate d_n at balanced load
+        d_nb = d_0 * (eps_cu) / (eps_sy + eps_cu)
 
-            # convert centroid to local coordinates
-            _, c_v = principal_coordinate(
-                phi=theta * 180 / np.pi, x=centroid[0], y=centroid[1]
-            )
-
-            # calculate d
-            _, ef_v = principal_coordinate(
-                phi=theta * 180 / np.pi,
-                x=extreme_fibre[0],
-                y=extreme_fibre[1],
-            )
-            d = ef_v - c_v
-
-            if d > d_0:
-                d_0 = d
-                extreme_geom = steel_geom
-
-        # 2) calculate yield strain
-        yield_strain = (
-            extreme_geom.material.stress_strain_profile.yield_strength
-            / extreme_geom.material.stress_strain_profile.elastic_modulus
-        )
-
-        # 3) k_uo at balanced load
-        k_uob = 0.003 / (0.003 + yield_strain)
-
-        # 4) calculate d_n at balanced load
-        d_nb = k_uob * d_0
-
-        # 5) calculate axial force at balanced load
+        # calculate axial force at balanced load
         balanced_res = self.concrete_section.calculate_ultimate_section_actions(
             d_n=d_nb, ultimate_results=res.UltimateBendingResults(theta=theta)
         )
@@ -528,29 +532,24 @@ class AS3600(DesignCode):
 
     def ultimate_bending_capacity(
         self,
-        theta: Optional[float] = 0,
-        n: Optional[float] = 0,
-        phi_0: Optional[float] = 0.6,
+        theta: float = 0,
+        n: float = 0,
+        phi_0: float = 0.6,
     ) -> Tuple[res.UltimateBendingResults, res.UltimateBendingResults, float]:
         r"""Calculates the ultimate bending capacity with capacity factors to
         AS 3600:2018.
 
         :param theta: Angle (in radians) the neutral axis makes with the horizontal axis
             (:math:`-\pi \leq \theta \leq \pi`)
-        :type theta: Optional[float]
         :param n: Net axial force
-        :type n: Optional[float]
         :param phi_0: Compression dominant capacity reduction factor, see Table 2.2.2(d)
-        :type phi_0: Optional[float]
 
         :return: Factored and unfactored ultimate bending results objects, and capacity
             reduction factor *(factored_results, unfactored_results, phi)*
-        :rtype: Tuple[:class:`~concreteproperties.results.UltimateBendingResults`,
-            :class:`~concreteproperties.results.UltimateBendingResults`, float]
         """
 
         # get parameters to determine phi
-        n_uot = self.concrete_section.gross_properties.tensile_load
+        n_uot = self.tensile_load
         k_uo = self.get_k_uo(theta=theta)
         n_ub = self.get_n_ub(theta=theta)
 
@@ -571,7 +570,7 @@ class AS3600(DesignCode):
             a=phi_0,
             b=0.85,
             xtol=1e-3,
-            rtol=1e-6,
+            rtol=1e-6,  # type: ignore
             full_output=True,
             disp=False,
         )
@@ -586,29 +585,60 @@ class AS3600(DesignCode):
         factored_ult_res.n *= phi
         factored_ult_res.m_x *= phi
         factored_ult_res.m_y *= phi
-        factored_ult_res.m_u *= phi
+        factored_ult_res.m_xy *= phi
 
         return factored_ult_res, ult_res, phi
 
     def moment_interaction_diagram(
         self,
-        phi_0: Optional[float] = 0.6,
-        **kwargs,
+        phi_0: float = 0.6,
     ) -> Tuple[res.MomentInteractionResults, res.MomentInteractionResults, List[float]]:
         """Generates a moment interaction diagram with capacity factors to AS 3600:2018.
 
         :param phi_0: Compression dominant capacity reduction factor, see Table 2.2.2(d)
-        :type phi_0: Optional[float]
-        :param kwargs: Keyword arguments passed to
-            :meth:`~concreteproperties.concrete_section.ConcreteSection.moment_interaction_diagram`
 
         :return: Factored and unfactored moment interaction results objects, and list of
             capacity reduction factors *(factored_results, unfactored_results, phis)*
-        :rtype: Tuple[:class:`~concreteproperties.results.MomentInteractionResults`,
-            :class:`~concreteproperties.results.MomentInteractionResults`, List[float]]
         """
 
-        mi_res = self.concrete_section.moment_interaction_diagram(**kwargs)
+        mi_res = self.concrete_section.moment_interaction_diagram(
+            control_points=[
+                ("D", 1.0),
+                ("fy", 1.0),
+                ("N", 0.0),
+            ],
+            n_points=[12, 12],
+        )
+
+        # get theta
+        theta = mi_res.results[0].theta
+
+        # add squash load
+        mi_res.results.insert(
+            0,
+            res.UltimateBendingResults(
+                theta=theta,
+                d_n=inf,
+                k_u=0,
+                n=self.squash_load,
+                m_x=0,
+                m_y=0,
+                m_xy=0,
+            ),
+        )
+
+        # add tensile load
+        mi_res.results.append(
+            res.UltimateBendingResults(
+                theta=theta,
+                d_n=0,
+                k_u=0,
+                n=self.tensile_load,
+                m_x=0,
+                m_y=0,
+                m_xy=0,
+            )
+        )
 
         # make a copy of the results to factor
         factored_mi_res = deepcopy(mi_res)
@@ -616,19 +646,12 @@ class AS3600(DesignCode):
         # list to store phis
         phis = []
 
-        # get required constants
-        n_uot = self.concrete_section.gross_properties.tensile_load
+        # get required constants for phi
+        n_uot = self.tensile_load
+        k_uo = self.get_k_uo(theta=theta)
+        n_ub = self.get_n_ub(theta=theta)
 
-        # positive bending
-        k_uo = mi_res.results[-2].k_u
-        n_ub = self.get_n_ub(theta=mi_res.results[0].theta)
-
-        # negative bending
-        if len(mi_res.results_neg) > 0:
-            k_uo_neg = mi_res.results_neg[-2].k_u
-            n_ub_neg = self.get_n_ub(theta=mi_res.results_neg[0].theta)
-
-        # factor results for positive bending
+        # factor results
         for ult_res in factored_mi_res.results:
             phi = self.capacity_reduction_factor(
                 n_u=ult_res.n, n_ub=n_ub, n_uot=n_uot, k_uo=k_uo, phi_0=phi_0
@@ -636,41 +659,25 @@ class AS3600(DesignCode):
             ult_res.n *= phi
             ult_res.m_x *= phi
             ult_res.m_y *= phi
-            ult_res.m_u *= phi
-            phis.append(phi)
-
-        # factor results for negative bending
-        for ult_res in factored_mi_res.results_neg:
-            phi = self.capacity_reduction_factor(
-                n_u=ult_res.n, n_ub=n_ub_neg, n_uot=n_uot, k_uo=k_uo_neg, phi_0=phi_0
-            )
-            ult_res.n *= phi
-            ult_res.m_x *= phi
-            ult_res.m_y *= phi
-            ult_res.m_u *= phi
+            ult_res.m_xy *= phi
             phis.append(phi)
 
         return factored_mi_res, mi_res, phis
 
     def biaxial_bending_diagram(
         self,
-        n: Optional[float] = 0,
-        n_points: Optional[int] = 48,
-        phi_0: Optional[float] = 0.6,
+        n: float = 0,
+        n_points: int = 48,
+        phi_0: float = 0.6,
     ) -> Tuple[res.BiaxialBendingResults, List[float]]:
         """Generates a biaxial bending with capacity factors to AS 3600:2018.
 
         :param n: Net axial force
-        :type n: Optional[float]
         :param n_points: Number of calculation points between the decompression
-        :type n_points: Optional[int]
         :param phi_0: Compression dominant capacity reduction factor, see Table 2.2.2(d)
-        :type phi_0: Optional[float]
 
         :return: Factored biaxial bending results object and list of capacity reduction
             factors *(factored_results, phis)*
-        :rtype: Tuple[:class:`~concreteproperties.results.BiaxialBendingResults`,
-            List[float]]
         """
 
         pass
@@ -697,7 +704,9 @@ class AS3600(DesignCode):
             # loop through thetas
             for theta in theta_list:
                 # factored capacity
-                f_ult_res, _, phi = self.ultimate_bending_capacity(theta=theta, n=n)
+                f_ult_res, _, phi = self.ultimate_bending_capacity(
+                    theta=theta, n=n, phi_0=phi_0
+                )
                 f_bb_res.results.append(f_ult_res)
                 phis.append(phi)
 
